import stamp.math.*;        // include Package stamp.math
import stamp.core.*;        // include Package stamp.core

/**
 * Program : ADS7841 Analog-to-Ditital Converter.
 * class   : ADS7841_4CH_V2
 * @version 1.0 01/05/2004
 * @author Prajin Palangsantikul
 */

public class ADS7841_4CH_V2 {

  final static char HOME  = 0x01;       // cursor to home
  final static char CLS = '\u0010';     // clear screen
  final static int  CLOCK_PIN = CPU.pin4;
  final static int  CS_PIN    = CPU.pin5;
  final static int  DATA_PIN  = CPU.pin6;
  static int [] CMD_CTL ={0x94, 0xD4, 0xA4, 0xE4};  // Control byte ADS7841 CH1-CH4

  static int readADS7841(int CH){
    int data;
    // enable chip set logic low
    CPU.writePin(CS_PIN,false);
    // send command
    CPU.shiftOut(DATA_PIN, CLOCK_PIN, 16, CPU.SHIFT_MSB, CMD_CTL[CH]);
    // received 12 bit data
    data = CPU.shiftIn(DATA_PIN, CLOCK_PIN, 12, CPU.POST_CLOCK_MSB);
    // disable chip set logic High
    CPU.writePin(CS_PIN,true);
    return(data);
  }

  public static void main() {

    int numCH=0;
    int volte;
    // Create a new Float32 object with initial value of zero.
    Float32 fnum1 = new Float32();
    Float32 fnum2 = new Float32();

    System.out.print(CLS);
    fnum1.set(".0012");   // fnum1 = 0.0012

    do {
        System.out.print("Analog-to-Ditital Converter.");   // Output
        System.out.print("Select Number CH (1-4) : ");   // Output
        switch (Terminal.getChar() ) {        // run code based on getChar
        case '1': numCH = 1; break;
        case '2': numCH = 2; break;
        case '3': numCH = 3; break;
        case '4': numCH = 4; break;
        default:                            // execute if no match above
          System.out.println("\nYou didn't enter 1-4!"); // Output
        }
    } while(numCH ==0 );

    while(true){
      System.out.print(HOME);
      System.out.print(CLS);
      for(int i=0; i<numCH; i++) {

        System.out.print("Data Ch.");
        System.out.print(i);
        System.out.print(" : ");
        volte = readADS7841(i);
        System.out.println(readADS7841(i));
        System.out.print("Volte: ");
        fnum2.set(volte);
        fnum2.multiply(fnum1);
        System.out.print(fnum2.toString());
        System.out.println(" V");
        CPU.delay(10000);
      }
    }
  }
}