import stamp.peripheral.memory.eeprom.*;  // include Package stamp.peripheral.memory.eeprom
import stamp.core.*;        // include Package stamp.core
import stamp.protocol.*;
/**
 * Program : 24LC256 eeprom
 * class   : test24LC256
 * @version 1.0 01/05/2004
 * @author Prajin Palangsantikul
 */

public class test24LC256 {
  public static void main() {
    final int sdaPin = CPU.pin11;  // Serial data pin
    final int sclPin = CPU.pin12;  // Serial clock pin
    MC24LC256 ee = new MC24LC256(sdaPin,sclPin);    // Create MC24LC256 object
    boolean test = true;                             // validate
    byte [] dat = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10};
    int num;                                         // value from eeprom

    System.out.println("Test : Write/Read 24LC256 eeprom\n");

    // Write one byte of value 45 to memory location 50 on chip 0
    System.out.println("Test 1: writeOne value 45 to location 50 on chip address 0");
    test = ee.writeOne(0,50,(byte) 45);
    if (test)
      System.out.println("Test 1: writeOne [OK]");
    else
      System.out.println("Test 1: writeOne [BAD]");

    // Read from location 50 from chip 0 and test for errors.
    System.out.println("Read from location 50 on chip address 0");
    try {
      num = ee.readRandom(0,50);
      System.out.print("Data read from location 50 : ");
      System.out.println(num);
    }// end try
    catch (MC24LC256BadReadException bre) {
      System.out.println("Bad Read Detected Here");
    }

    // Test of the array write -- starting at 1st memory location
    System.out.print("\n");
    System.out.println("Test 2: writePage value 1-10 to location 0-9 on chip address 0");
    test = ee.writePage(0, 0, dat);
    CPU.delay(100);
    if (test)
      System.out.println("Test 2: writePage [OK]");
    else
      System.out.println("Test 2: writePage [BAD]");

    try {
      System.out.println("Data read from location 0-9 on chip address 0");
      for(int i=0; i<10; i++) {
        num = ee.readRandom(0, i);
        System.out.print("Data read from location ");
        System.out.print(i);
        System.out.print(" : ");
        System.out.println(num);
      }
    }// end try
    catch (MC24LC256BadReadException bre) {
      System.out.println("Bad Read Detected Here");
    }


  }
}