import stamp.core.*;        // include Package stamp.core
import stamp.protocol.*;    // include Package stamp.protocol

/**
 * Program : DS1307 Real Time Clock
 * class   : testDS1307
 * @version 1.0 01/05/2004
 * @author Prajin Palangsantikul
 */

class DS1307 {

  static int DS1307_ID;
  static I2C i2cbus;    // reference to an object
  static StringBuffer sbuf = new StringBuffer(10);

  static int hour, min, sec;
  static int day, date, month, year;

  // constructor: DS1307
  DS1307(int id, int SDAPin, int SCLPin) {
    DS1307_ID = id;
    i2cbus = new I2C(SDAPin, SCLPin); //create object i2cbus (or instantiate)
  }

  // toHex
  static char toHex(int num) {
    char hexDigit;

    hexDigit = (char) ('0' + (num & 0x0F));
    if (hexDigit > '9') hexDigit += 7;
      return(hexDigit);
  }

  // NumToHex
  static String NumToHex(int num) {
    sbuf.clear();
    sbuf.append(toHex(num >> 4));
    sbuf.append(toHex(num));
    return (sbuf.toString());
  }

  // AssignToHex
  static int AssignToHex(int hi, int lo) {
    return((hi<<4)|lo);
  }

  //read DS1307
  static int read(int addr) {
    int dat;

    i2cbus.start();
    i2cbus.write(DS1307_ID);
    i2cbus.write(addr);
    i2cbus.start();
    i2cbus.write(DS1307_ID+1);
    dat = i2cbus.read(1);
    i2cbus.stop();
    return(dat);
  }

  // Read Time from DS1307
  static void ReadTime() {
    sec   = read(0x00);
    min   = read(0x01);
    hour  = read(0x02);
  }

  // Write Time to DS1307
  static void WriteTime(int hh, int mm, int ss) {
    i2cbus.start();
    i2cbus.write(DS1307_ID);
    i2cbus.write(0x00);
    i2cbus.write(ss);
    i2cbus.write(mm);
    i2cbus.write(hh);
    i2cbus.stop();
  }

  // Read Date from DS1307
  static void ReadDate() {
    date  = read(0x04);
    month = read(0x05);
    year  = read(0x06);
  }

  // Write Date to DS1307
  static void WriteDate(int dd, int mm, int yy) {
    i2cbus.start();
    i2cbus.write(DS1307_ID);
    i2cbus.write(0x04);
    i2cbus.write(dd);
    i2cbus.write(mm);
    i2cbus.write(yy);
    i2cbus.stop();
  }
}

public class testDS1307 {

  final static char CLS = '\u0010';     // clear screen
  final public static int DS1307_ADDRESS  = 0xD0;
  final public static int SDAPin = CPU.pin11;
  final public static int SCLPin = CPU.pin12;
  static StringBuffer sBuf1 = new StringBuffer(20);
  static Integer intNum = new Integer(0);
  //create object RTC
  public static DS1307 RTC = new DS1307(DS1307_ADDRESS,SDAPin,SCLPin);

  // Read char to Stringbuffer
  public static void ReadCharToStringbuffer() {
    char data;

    while ( (data = Terminal.getChar()) != '\r' ) {
      sBuf1.append(data);
    }
    System.out.print('\n');
  }

  // wait char
  public static char getC() {
    return(Terminal.getChar());
  }

  //
  static void DisplayDateTime() {
    RTC.ReadTime();
    RTC.ReadDate();

    System.out.print("Date : ");
    System.out.print(RTC.NumToHex(RTC.date));
    System.out.print("/");
    System.out.print(RTC.NumToHex(RTC.month));
    System.out.print("/");
    System.out.print(RTC.NumToHex(RTC.year));
    System.out.print(" Time : ");
    System.out.print(RTC.NumToHex(RTC.hour));
    System.out.print(":");
    System.out.print(RTC.NumToHex(RTC.min));
    System.out.print(":");
    System.out.print(RTC.NumToHex(RTC.sec));
  }

  public static void SetTime() {
    int hh,mm,ss;

    System.out.println("\nSet Time : hh:mm:ss");
    System.out.print("Input hh : ");
    ReadCharToStringbuffer();
    System.out.print("Input mm : ");
    ReadCharToStringbuffer();
    System.out.print("Input ss : ");
    ReadCharToStringbuffer();
    hh = RTC.AssignToHex(intNum.parseInt(sBuf1.subString(0, 1)),
                      intNum.parseInt(sBuf1.subString(1, 2)));
    mm = RTC.AssignToHex(intNum.parseInt(sBuf1.subString(2, 3)),
                      intNum.parseInt(sBuf1.subString(3, 4)));
    ss = RTC.AssignToHex(intNum.parseInt(sBuf1.subString(4, 5)),
                      intNum.parseInt(sBuf1.subString(5, 6)));
    RTC.WriteTime(hh,mm,ss);   // write time hh:mm:ss
//    RTC.WriteTime(0x04, 0x48, 0x00);   // write time hh:mm:ss
  }

  public static void SetDate() {
    int dd,mm,yy;

    System.out.println("\nSet Date : dd/mm/yy");
    System.out.print("Input dd : ");
    ReadCharToStringbuffer();
    System.out.print("Input mm : ");
    ReadCharToStringbuffer();
    System.out.print("Input yy : ");
    ReadCharToStringbuffer();
    dd = RTC.AssignToHex(intNum.parseInt(sBuf1.subString(0, 1)),
                  intNum.parseInt(sBuf1.subString(1, 2)));
    mm = RTC.AssignToHex(intNum.parseInt(sBuf1.subString(2, 3)),
                  intNum.parseInt(sBuf1.subString(3, 4)));
    yy = RTC.AssignToHex(intNum.parseInt(sBuf1.subString(4, 5)),
                  intNum.parseInt(sBuf1.subString(5, 6)));
    RTC.WriteDate(dd,mm,yy);   // write date dd/mm/yy
//    RTC.WriteDate(0x12, 0x06, 0x04);   // write date dd/mm/yy
  }

  public static void MenuMessage() {
    System.out.println("Valid commands are:");
    System.out.println("  t - set time");
    System.out.println("  d - set date");
    System.out.println("  Press Any key display date/time");
    System.out.print("\n> ");
  }

  public static void main() {
    char ch;

    System.out.println("\u0010DS1307 Real Time Clock\n");
    MenuMessage();
    ch = getC();
    if(ch == 't') {
      SetTime();
    }
    else if(ch == 'd') {
      SetDate();
    }
    while(true) {
      System.out.print(CLS);
      DisplayDateTime();
      CPU.delay(10000);
    }
  }
}