import stamp.core.*;        // include Package stamp.core
import stamp.protocol.*;    // include Package stamp.protocol

/**
 * Program : DS1307 Real Time Clock
 * class   : testDS1307
 * @version 1.0 01/05/2004
 * @author Prajin Palangsantikul
 */

class SLCD {

  final static int SLCD = CPU.pin13;  // connect port
  final static int CMD = 0xFE;        // standard mode command (ST)
  // //create object txUart
  static Uart txUart = new Uart( Uart.dirTransmit, SLCD, Uart.invert,
                                 Uart.speed9600, Uart.stop1 );

  // Send Command to SLCD
  static void SendCmd(int cmd) {
    txUart.sendByte(CMD);     // send ST command
    txUart.sendByte(cmd);     // send command
  }

  // Show message to SLCD line #1
  static void showLine1(String s) {
    SendCmd(0x80);            // set line 1
    txUart.sendByte(CMD);     // send ST command
    txUart.sendByte(1);       // send 1 display message
    txUart.sendString(s);     // send message
  }

  // Show message to SLCD line #2
  static void showLine2(String s) {
    SendCmd(0xC0);            // set line 2
    SendCmd(0x10);            // move cursor to left
    SendCmd(0x10);
    txUart.sendByte(0);       // send ST command
    txUart.sendByte(1);       // send 1 display message
    txUart.sendString(s);     // send message
  }
}

class DS1307 {

  static int DS1307_ID;
  static I2C i2cbus;    // reference to an object
  static StringBuffer sbuf = new StringBuffer(10);

  static int hour, min, sec;
  static int day, date, month, year;

  // constructor: DS1307
  DS1307(int id, int SDAPin, int SCLPin) {
    DS1307_ID = id;
    i2cbus = new I2C(SDAPin, SCLPin); //create object i2cbus (or instantiate)
  }

  // toHex
  static char toHex(int num) {
    char hexDigit;

    hexDigit = (char) ('0' + (num & 0x0F));
    if (hexDigit > '9') hexDigit += 7;
      return(hexDigit);
  }

  // NumToHex
  static String NumToHex(int num) {
    sbuf.clear();
    sbuf.append(toHex(num >> 4));
    sbuf.append(toHex(num));
    return (sbuf.toString());
  }

  // AssignToHex
  static int AssignToHex(int hi, int lo) {
    return((hi<<4)|lo);
  }

  //read DS1307
  static int read(int addr) {
    int dat;

    i2cbus.start();
    i2cbus.write(DS1307_ID);
    i2cbus.write(addr);
    i2cbus.start();
    i2cbus.write(DS1307_ID+1);
    dat = i2cbus.read(1);
    i2cbus.stop();
    return(dat);
  }

  // Read Time from DS1307
  static void ReadTime() {
    sec   = read(0x00);
    min   = read(0x01);
    hour  = read(0x02);
  }

  // Write Time to DS1307
  static void WriteTime(int hh, int mm, int ss) {
    i2cbus.start();
    i2cbus.write(DS1307_ID);
    i2cbus.write(0x00);
    i2cbus.write(ss);
    i2cbus.write(mm);
    i2cbus.write(hh);
    i2cbus.stop();
  }

  // Read Date from DS1307
  static void ReadDate() {
    date  = read(0x04);
    month = read(0x05);
    year  = read(0x06);
  }

  // Write Date to DS1307
  static void WriteDate(int dd, int mm, int yy) {
    i2cbus.start();
    i2cbus.write(DS1307_ID);
    i2cbus.write(0x04);
    i2cbus.write(dd);
    i2cbus.write(mm);
    i2cbus.write(yy);
    i2cbus.stop();
  }
}

public class testDS1307 {

  final public static int DS1307_ADDRESS  = 0xD0;
  final public static int SDAPin = CPU.pin11;
  final public static int SCLPin = CPU.pin12;
  static StringBuffer sBuf1 = new StringBuffer(20);
  static StringBuffer sBuf2 = new StringBuffer(20);
  //create object RTC
  public static DS1307 RTC = new DS1307(DS1307_ADDRESS,SDAPin,SCLPin);
  //create object slcd
  public static SLCD slcd = new SLCD();

  static void DisplayDateTime() {
    RTC.ReadDate();
    RTC.ReadTime();

    sBuf1.clear();
    sBuf2.clear();
    sBuf1.append("Date : ");
    sBuf1.append(RTC.NumToHex(RTC.date));
    sBuf1.append("/");
    sBuf1.append(RTC.NumToHex(RTC.month));
    sBuf1.append("/");
    sBuf1.append(RTC.NumToHex(RTC.year));
    sBuf2.append("Time : ");
    sBuf2.append(RTC.NumToHex(RTC.hour));
    sBuf2.append(":");
    sBuf2.append(RTC.NumToHex(RTC.min));
    sBuf2.append(":");
    sBuf2.append(RTC.NumToHex(RTC.sec));
    slcd.showLine1(sBuf1.toString());
    slcd.showLine2(sBuf2.toString());
  }

  public static void main() {

    slcd.SendCmd(0x38);   // set LCD 2nd line
    while(true) {
      DisplayDateTime();
      CPU.delay(10000);
    }
  }
}